#include "Main.h"
#include "InfoWindow.h"

#include "Prefs.h"
#include "View.h"
#include "Weather.h"
#include <TabView.h>
#include <Message.h>
#include <Messenger.h>
#include <ScrollView.h>
#include <StringView.h>
#include <TextView.h>

WInfoWindow::WInfoWindow(WPrefs *iprefs, WWeather *iweather, BMessenger *imsger)
	: BWindow(iprefs->opInfoRect,
		APP_NAME" Info", B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE),
	prefs(iprefs),
	weather(iweather),
	msger(imsger)
{
	BView *cover = new BView(Bounds(), "",
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW);
	cover->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	AddChild(cover);
	
	BRect sr(Bounds());
	//sr.InsetBy(10, 10);
	sr.InsetBy(-1, -1);
	sr.top += 5;
	
	BTabView *tab = new BTabView(sr, "", B_WIDTH_FROM_WIDEST);
	cover->AddChild(tab);
	
	//add conditions tab
	sr = tab->Bounds();
	sr.bottom -= tab->TabHeight();
	cover = new BView(sr, "Current Conditions",
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW);
	cover->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	tab->AddTab(cover);
	tab->Select(0);
	
	sr = cover->Bounds();
	sr.InsetBy(10, 10);
	sr.bottom = sr.top + be_plain_font->Size() + 1;
	
#define ADDFIELD(WORD, VAR) \
{\
	/*"Last Updated:" is the longest string used*/\
	float center = cover->StringWidth("Temperature:");\
	BRect r(sr);\
	r.left += center - cover->StringWidth(WORD);\
	cover->AddChild(new BStringView(r, "", WORD,\
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP));\
	r = sr;\
	r.left += center + 5;\
	VAR = new BStringView(r, "", "",\
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP);\
	cover->AddChild(VAR);\
	sr.OffsetBy(0, VAR->Bounds().bottom + 19 - be_plain_font->Size());\
}
	ADDFIELD("Location:", vLoc);
	ADDFIELD("METAR Time:", vTime);
	ADDFIELD("Temperature:", vTemp);
	ADDFIELD("Dew:", vDew);
	ADDFIELD("Sky:", vSky);
	ADDFIELD("Conditions:", vCond);
	ADDFIELD("Wind:", vWind);
	ADDFIELD("Visibility:", vVis);
	ADDFIELD("Pressure:", vPres);
	ADDFIELD("Humidity:", vHum);
#undef ADDFIELD
	
	//add forecast tab
	sr = tab->Bounds();
	sr.bottom -= tab->TabHeight();
	cover = new BView(sr, "Forecast",
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW);
	cover->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	tab->AddTab(cover);
	
	sr = cover->Bounds();
	sr.InsetBy(10, 10);
	sr.right -= B_V_SCROLL_BAR_WIDTH;
	BRect ur(sr);
	ur.OffsetTo(0, 0);
	ur.InsetBy(3, 3);
	vFore = new BTextView(sr, "", ur,
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW);
	cover->AddChild(new BScrollView("", vFore,
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM, B_FULL_UPDATE_ON_RESIZE, false, true));
	vFore->MakeEditable(false);
	
	Update();
}

bool WInfoWindow::QuitRequested(void)
{
	if(msger)
	{
		msger->SendMessage(WI_CLOSE);
		return false;
	}
	else
		return true;
}

void WInfoWindow::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
		case WI_UPDATE:
			Update();
			break;
		case WI_CLOSE:
			delete msger;
			msger = 0;
			SavePos();
			message->SendReply(WI_REPLY);
			PostMessage(B_QUIT_REQUESTED);
			break;
		case WI_ACTIVATE:
			Activate();
			break;
		
		default:
			BWindow::MessageReceived(message);
			break;
	}
}

void WInfoWindow::SavePos(void)
{
	prefs->sem.get();
	prefs->opInfoRect = Frame();
	prefs->sem.rel();
}

void WInfoWindow::Update(void)
{
	weather->sem.get();
	
	vLoc->SetText(weather->GetLocStr());
	vTime->SetText(weather->GetTimeStr());
	vTemp->SetText(weather->GetTempStr());
	vDew->SetText(weather->GetDewStr());
	vSky->SetText(weather->GetSkyStr());
	vCond->SetText(weather->GetCondStr());
	vWind->SetText(weather->GetWindStr());
	vVis->SetText(weather->GetVisStr());
	vPres->SetText(weather->GetPresStr());
	vHum->SetText(weather->GetHumStr());
	
	vFore->SetText(weather->GetForeStr());
	
	weather->sem.rel();
}