#include "Main.h"
#include "Locations.h"

#include <Entry.h>
#include <File.h>
#include <FindDirectory.h>
#include <Path.h>

extern char *LocationString;

WLocations::WLocations(void)
{
	head = new WLocation;
	head->name[0] = 0;
	head->code[0] = 0;
	head->zone[0] = 0;
	head->radar[0] = 0;
	head->child = 0;
	head->sibling = 0;
	
	char *string;
	BPath path;
	find_directory(B_USER_SETTINGS_DIRECTORY, &path);
	path.Append("BWeather_locations");
	BFile file;
	if(file.SetTo(path.Path(), B_READ_ONLY) != B_OK)
	{
		string = new char[strlen(LocationString) + 1];
		strcpy(string, LocationString);
		//Locations.txt doesn't exist, so fill it with the default locations
		if(file.SetTo(path.Path(), B_READ_WRITE | B_CREATE_FILE) == B_OK)
		{
			//we don't NEED to create locations.txt...
			file.SetSize(strlen(LocationString) + 1);
			file.SetSize(strlen(LocationString) + 1);
			file.Write(LocationString, strlen(LocationString) + 1);
		}
	}
	else
	{
		off_t size;
		file.GetSize(&size);
		string = new char[size];
		file.Read(string, size);
	}
	
	LoadLoc(string, head);
	delete[] string;
}

WLocations::~WLocations(void)
{
	DeleteLoc(head);
}

void WLocations::DeleteLoc(WLocation *loc)
{
	WLocation *cur, *tmp;
	cur = loc->child;
	while(cur)
	{
		tmp = cur->sibling;
		DeleteLoc(cur);
		cur = tmp;
	}
	delete loc;
}

void WLocations::LoadLoc(char *string, WLocation *loc)
{
	char line[1024], *cur;
	WLocation *newLoc, *lastLoc = 0;
	
	//get code
	string = GetLine(string, line);
	memcpy(loc->code, line + 1, strlen(line) - 2);
	loc->code[strlen(line) - 2] = 0;
	
	bool done = false;
	while(!done)
	{
		string = GetLine(string, line);
		if(strstr(line, "name="))
		{
			strcpy(loc->name, line + strlen("name="));
		}
		else if(strstr(line, "regions=") || strstr(line, "states="))
		{
			if(line[0] == 'r')
				cur = line + strlen("regions=");
			else
				cur = line + strlen("states=");
				
			char codeBuf[256];
			int s, i;
			if(strcmp(loc->code, "Main"))
				sprintf(codeBuf, "[%s_", loc->code);
			else
				sprintf(codeBuf, "[");
			s = strlen(codeBuf);
			i = s;
			while(cur)
			{
				//at a 0 or a ' ' we have a complete code, so add it as a child
				if(*cur == 0 || *cur == ' ')
				{
					newLoc = new WLocation;
					newLoc->name[0] = 0;
					newLoc->code[0] = 0;
					newLoc->zone[0] = 0;
					newLoc->radar[0] = 0;
					newLoc->child = 0;
					newLoc->sibling = 0;
					
					if(lastLoc)
					{
						lastLoc->sibling = newLoc;
						lastLoc = newLoc;
					}
					else
					{
						loc->child = newLoc;
						lastLoc = newLoc;
					}
					
					codeBuf[i] = ']';
					codeBuf[i + 1] = 0;
					LoadLoc(strstr(string, codeBuf), newLoc);
					i = s;
					if(*cur == 0)
						cur = 0;
					else
						cur++;
				}
				else
				{
					codeBuf[i] = *cur;
					i++;
					cur++;
				}
			}
		}
		else if(strstr(line, "loc"))
		{
			cur = strstr(line, "=") + 1;
			newLoc = new WLocation;
			cur = GetWord(cur, newLoc->name);
			cur = GetWord(cur, newLoc->code);
			cur = GetWord(cur, newLoc->zone);
			cur = GetWord(cur, newLoc->radar);
			newLoc->child = 0;
			newLoc->sibling = 0;
			if(lastLoc)
			{
				lastLoc->sibling = newLoc;
				lastLoc = newLoc;
			}
			else
			{
				loc->child = newLoc;
				lastLoc = newLoc;
			}
		}
		else
			done = true;
	}
}

char *WLocations::GetWord(char *start, char *buf)
{
	int size;
	char *end = strstr(start, " ");
	if(end)
	{
		while(*(end - 1) == '\\')
			end = strstr(end + 1, " ");
		size = int(end) - int(start);
	}
	else
		size = strlen(start);
		
	memcpy(buf, start, size);
	buf[size] = 0;
	
	//replace '\\ ' with ' '
	end = strstr(buf, " ");
	while(end)
	{
		for(uint i=0; i<=strlen(end); i++)
			end[i - 1] = end[i];
		end = strstr(end, " ");
	}
	
	return start + size + 1;
}

char *WLocations::GetLine(char *start, char *buf)
{
	if(strcmp(start, ""))
	{
		char *end = strstr(start, "\n");
		int size = int(end) - int(start);
		memcpy(buf, start, size);
		buf[size] = 0;
		return start + size + 1;
	}
	else
	{
		buf[0] = 0;
		return start;
	}
}