#include "Main.h"
#include "Weather.h"

#include "Prefs.h"
#include "View.h"
#include "HTTP_IO.h"
#include <Bitmap.h>
#include <Looper.h>
#include <Handler.h>
#include <Message.h>
#include <OS.h>
#include <Resources.h>
#include <Roster.h>
#include <String.h>
#include <StorageDefs.h>

typedef struct httpThreadData
{
	char metar[SERVER_MAX_LEN + 1], iwin[SERVER_MAX_LEN + 1];
	char code[LOCATION_CODE_LEN + 1], zone[LOCATION_ZONE_LEN + 1];
	WWeather *weather;
	BHandler *handler;
};

int32 httpThreadFunc(httpThreadData *data);
void parse_metar(WWeather *weather, char *code, char *string);
bool metar_tok_time(WWeather *weather, char *tok);
bool metar_tok_temp(WWeather *weather, char *tok);
bool metar_tok_sky(WWeather *weather, char *tok);
bool metar_tok_cond(WWeather *weather, char *tok);
bool metar_tok_wind(WWeather *weather, char *tok);
bool metar_tok_vis(WWeather *weather, char *tok);
bool metar_tok_pres(WWeather *weather, char *tok);
void parse_iwin(WWeather *weather, char *zone, char *string);
bool iwin_range_match(char *range, char *zone);

WWeather::WWeather(WPrefs *iprefs, BHandler *ihandler)
	: sem("weather sem"),
	prefs(iprefs),
	handler(ihandler)
{
	regcomp(&metar_re[TIME_RE], TIME_RE_STR, REG_EXTENDED);
	regcomp(&metar_re[WIND_RE], WIND_RE_STR, REG_EXTENDED);
	regcomp(&metar_re[VIS_RE], VIS_RE_STR, REG_EXTENDED);
	regcomp(&metar_re[CLOUD_RE], CLOUD_RE_STR, REG_EXTENDED);
	regcomp(&metar_re[TEMP_RE], TEMP_RE_STR, REG_EXTENDED);
	regcomp(&metar_re[PRES_RE], PRES_RE_STR, REG_EXTENDED);
	regcomp(&metar_re[COND_RE], COND_RE_STR, REG_EXTENDED);
	regcomp(&iwin_re, IWIN_RE_STR, REG_EXTENDED);
	
	updating = true;
	
	BlankInfo();
}

WWeather::~WWeather(void)
{
	sem.get();
	
	regfree(&metar_re[TIME_RE]);
	regfree(&metar_re[WIND_RE]);
	regfree(&metar_re[VIS_RE]);
	regfree(&metar_re[CLOUD_RE]);
	regfree(&metar_re[TEMP_RE]);
	regfree(&metar_re[PRES_RE]);
	regfree(&metar_re[COND_RE]);
	regfree(&iwin_re);
	
	delete[] fore;
	
	thread_id thread = find_thread(WEATHER_THREAD);
	if(thread != B_NAME_NOT_FOUND)
		kill_thread(thread);
}

void WWeather::BlankInfo(void)
{
	time_str[0] = 0;
	temp = VALUE_NONE;
	dew = VALUE_NONE;
	sky = SKY_NONE;
	phem = PHENOMENON_NONE;
	qual = QUALIFIER_NONE;
	winddir = WIND_NONE;
	windspeed = VALUE_NONE;
	vis = VALUE_NONE;
	pres = VALUE_NONE;
	hum = VALUE_NONE;
	fore = new char[1];
	fore[0] = 0;
}

BBitmap *WWeather::ChooseBitmap(void)
{
	entry_ref ref;
	be_roster->FindApp(APP_SIGNATURE, &ref);
	BFile file(&ref, B_READ_ONLY);
	if(file.InitCheck() != B_OK)
		alert("Cannot load icons");
	//BFile file;
	//status_t err = file.SetTo(&ref, B_READ_ONLY);
	//status_t err = file.SetTo(&ref, B_READ_WRITE);
	BResources res(&file);
	/*if(!res.InitCheck())
		alert("Cannot load icons");*/
		
	//err = res.SetTo(&file);
	size_t size;
	BMessage arch;
	
	int id;
	id = RES_BLANK;
	
	switch(sky)
	{
		case SKY_NONE:
			break;
		case SKY_CLEAR:
			id = RES_SUN;
			break;
		case SKY_BROKEN:
		case SKY_SCATTERED:
		case SKY_FEW:
			id = RES_PART;
			break;
		case SKY_OVERCAST:
			id = RES_CLOUD;
			break;
	}
	switch(phem)
	{
		case PHENOMENON_NONE:
			break;
		case PHENOMENON_HAIL:
		case PHENOMENON_SMALL_HAIL:
		case PHENOMENON_DRIZZLE:
		case PHENOMENON_RAIN:
		case PHENOMENON_UNKNOWN_PRECIPITATION:
			id = RES_RAIN;
			break;
		case PHENOMENON_SNOW:
		case PHENOMENON_SNOW_GRAINS:
		case PHENOMENON_ICE_CRYSTALS:
		case PHENOMENON_ICE_PELLETS:
			id = RES_SNOW;
			break;
		case PHENOMENON_TORNADO:
		case PHENOMENON_SQUALL:
			id = RES_LIGHT;
			break;
		case PHENOMENON_MIST:
		case PHENOMENON_FOG:
		case PHENOMENON_SMOKE:
		case PHENOMENON_VOLCANIC_ASH:
		case PHENOMENON_SAND:
		case PHENOMENON_HAZE:
		case PHENOMENON_SPRAY:
		case PHENOMENON_DUST:
		case PHENOMENON_SANDSTORM:
		case PHENOMENON_DUSTSTORM:
		case PHENOMENON_FUNNEL_CLOUD:
		case PHENOMENON_DUST_WHIRLS:
			id = RES_CLOUD;
			break;
	}
	switch(qual)
	{
		case QUALIFIER_NONE:
		case QUALIFIER_VICINITY:
			break;
		case QUALIFIER_THUNDERSTORM:
			id = RES_LIGHT;
			break;
		case QUALIFIER_SHALLOW:
		case QUALIFIER_PATCHES:
		case QUALIFIER_PARTIAL:
		case QUALIFIER_BLOWING:
		case QUALIFIER_SHOWERS:
		case QUALIFIER_DRIFTING:
		case QUALIFIER_FREEZING:
			break;
		case QUALIFIER_LIGHT:
		case QUALIFIER_MODERATE:
		case QUALIFIER_HEAVY:
			break;
	}
	if(updating)
		id = RES_UPDATE;
	
	if(prefs->opInDeskbar)
		id++;
		
	char *loaded = (char*)res.LoadResource(type_code('BBMP'), id, &size);
	arch.Unflatten(loaded);
	BBitmap *bmp = new BBitmap(&arch);
	
	return bmp;
}

void WWeather::Update(void)
{
	if(find_thread(WEATHER_THREAD) == B_NAME_NOT_FOUND && strcmp(prefs->opLocCode, "----"))
	{
		prefs->sem.get();
		//spawn a thread to do the update
		
		httpThreadData *data = new httpThreadData;
		sprintf(data->metar, "%s%s", prefs->opMetarServer, prefs->opLocCode);
		sprintf(data->iwin, "%s", prefs->opIwinServer);
		strncat(data->iwin, prefs->opLocZone, 2);
		if(prefs->opDetailed)
			strcat(data->iwin, "/zone.html");
		else
			strcat(data->iwin, "/state.html");
		strcpy(data->code, prefs->opLocCode);
		strcpy(data->zone, prefs->opLocZone);
		data->weather = this;
		data->handler = handler;
		prefs->sem.rel();
		thread_id thread = spawn_thread((long(*)(void*))httpThreadFunc, WEATHER_THREAD, B_NORMAL_PRIORITY, data);
		resume_thread(thread);
	}
}

char *sky_str[] = {
	("No Information"),
    ("Clear"),
    ("Broken clouds"),
    ("Scattered clouds"),
    ("Few clouds"),
    ("Overcast")
};

char *cond_str[24][13] = {
/*                   NONE                         VICINITY                             LIGHT                      MODERATE                      HEAVY                      SHALLOW                      PATCHES                         PARTIAL                      THUNDERSTORM                    BLOWING                      SHOWERS                         DRIFTING                      FREEZING                      */
/*               *******************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************/
/* NONE          */ {"None",                          "None",                                  "None",                        "None",                           "None",                        "None",                          "None",                             "None",                          "None",                             "None",                          "None",                             "None",                           "None",                          },
/* DRIZZLE       */ {("Drizzle"),               ("Drizzle in the vicinity"),       ("Light drizzle"),       ("Moderate drizzle"),       ("Heavy drizzle"),       ("Shallow drizzle"),       ("Patches of drizzle"),       ("Partial drizzle"),       ("Thunderstorm"),             ("Windy drizzle"),         ("Showers"),                  ("Drifting drizzle"),       ("Freezing drizzle")       },
/* RAIN          */ {("Rain "),                 ("Rain in the vicinity") ,         ("Light rain"),          ("Moderate rain"),          ("Heavy rain"),          ("Shallow rain"),          ("Patches of rain"),          ("Partial rainfall"),      ("Thunderstorm"),             ("Blowing rainfall"),      ("Rain showers"),             ("Drifting rain"),          ("Freezing rain")          },
/* SNOW          */ {("Snow"),                  ("Snow in the vicinity") ,         ("Light snow"),          ("Moderate snow"),          ("Heavy snow"),          ("Shallow snow"),          ("Patches of snow"),          ("Partial snowfali"),      ("Snowstorm"),                ("Blowing snowfall"),      ("Snow showers"),             ("Drifting snow"),          ("Freezing snow")          },
/* SNOW_GRAINS   */ {("Snow grains"),           ("Snow grains in the vicinity") ,  ("Light snow grains"),   ("Moderate snow grains"),   ("Heavy snow grains"),   ("Shallow snow grains"),   ("Patches of snow grains"),   ("Partial snow grains"),   ("Snowstorm"),                ("Blowing snow grains"),   ("Snow grain showers"),       ("Drifting snow grains"),   ("Freezing snow grains")   },
/* ICE_CRYSTALS  */ {("Ice crystals"),          ("Ice crystals in the vicinity") , ("Few ice crystals"),    ("Moderate ice crystals"),  ("Heavy ice crystals"),  "??",                        ("Patches of ice crystals"),  ("Partial ice crystals"),  ("Ice crystal storm"),        ("Blowing ice crystals"),  ("Showers of ice crystals"),  ("Drifting ice crystals"),  ("Freezing ice crystals")  },
/* ICE_PELLETS   */ {("Ice pellets"),           ("Ice pellets in the vicinity") ,  ("Few ice pellets"),     ("Moderate ice pellets"),   ("Heavy ice pellets"),   ("Shallow ice pellets"),   ("Patches of ice pellets"),   ("Partial ice pellets"),   ("Ice pellet storm"),         ("Blowing ice pellets"),   ("Showers of ice pellets"),   ("Drifting ice pellets"),   ("Freezing ice pellets")   },
/* HAIL          */ {("Hail"),                  ("Hail in the vicinity") ,         ("Light hail"),          ("Moderate hail"),          ("Heavy hail"),          ("Shallow hail"),          ("Patches of hail"),          ("Partial hail"),          ("Hailstorm"),                ("Blowing hail"),          ("Hail showers"),             ("Drifting hail"),          ("Freezing hail")          },
/* SMALL_HAIL    */ {("Small hail"),            ("Small hail in the vicinity") ,   ("Light hail"),          ("Moderate small hail"),    ("Heavy small hail"),    ("Shallow small hail"),    ("Patches of small hail"),    ("Partial small hail"),    ("Small hailstorm"),          ("Blowing small hail"),    ("Showers of small hail"),    ("Drifting small hail"),    ("Freezing small hail")    },
/* PRECIPITATION */ {("Unknown precipitation"), ("Precipitation in the vicinity"), ("Light precipitation"), ("Moderate precipitation"), ("Heavy precipitation"), ("Shallow precipitation"), ("Patches of precipitation"), ("Partial precipitation"), ("Unknown thunderstorm"),     ("Blowing precipitation"), ("Showers, type unknown"),    ("Drifting precipitation"), ("Freezing precipitation") },
/* MIST          */ {("Mist"),                  ("Mist in the vicinity") ,         ("Light mist"),          ("Moderate mist"),          ("Thick mist"),          ("Shallow mist"),          ("Patches of mist"),          ("Partial mist"),          "??",                           ("Mist with wind"),        "??",                           ("Drifting mist"),          ("Freezing mist")          },
/* FOG           */ {("Fog"),                   ("Fog in the vicinity") ,          ("Light fog"),           ("Moderate fog"),           ("Thick fog"),           ("Shallow fog"),           ("Patches of fog"),           ("Partial fog"),           "??",                           ("Fog with wind"),         "??",                           ("Drifting fog"),           ("Freezing fog")           },
/* SMOKE         */ {("Smoke"),                 ("Smoke in the vicinity") ,        ("Thin smoke"),          ("Moderate smoke"),         ("Thick smoke"),         ("Shallow smoke"),         ("Patches of smoke"),         ("Partial smoke"),         ("Smoke w/ thunders"),        ("Smoke with wind"),       "??",                           ("Drifting smoke"),         "??"                         },
/* VOLCANIC_ASH  */ {("Volcanic ash"),          ("Volcanic ash in the vicinity") , "??",                      ("Moderate volcanic ash"),  ("Thick volcanic ash"),  ("Shallow volcanic ash"),  ("Patches of volcanic ash"),  ("Partial volcanic ash"),  ("Volcanic ash w/ thunders"), ("Blowing volcanic ash"),  ("Showers of volcanic ash "), ("Drifting volcanic ash"),  ("Freezing volcanic ash")  },
/* SAND          */ {("Sand"),                  ("Sand in the vicinity") ,         ("Light sand"),          ("Moderate sand"),          ("Heavy sand"),          "??",                        ("Patches of sand"),          ("Partial sand"),          "??",                           ("Blowing sand"),          "None",                             ("Drifting sand"),          "??"                         },
/* HAZE          */ {("Haze"),                  ("Haze in the vicinity") ,         ("Light haze"),          ("Moderate haze"),          ("Thick haze"),          ("Shallow haze"),          ("Patches of haze"),          ("Partial haze"),          "??",                           ("Haze with wind"),        "??",                           ("Drifting haze"),          ("Freezing haze")          },
/* SPRAY         */ {("Sprays"),                ("Sprays in the vicinity") ,       ("Light sprays"),        ("Moderate sprays"),        ("Heavy sprays"),        ("Shallow sprays"),        ("Patches of sprays"),        ("Partial sprays"),        "??",                           ("Blowing sprays"),        "??",                           ("Drifting sprays"),        ("Freezing sprays")        },
/* DUST          */ {("Dust"),                  ("Dust in the vicinity") ,         ("Light dust"),          ("Moderate dust"),          ("Heavy dust"),          "??",                        ("Patches of dust"),          ("Partial dust"),          "??",                           ("Blowing dust"),          "??",                           ("Drifting dust"),          "??"                         },
/* SQUALL        */ {("Squall"),                ("Squall in the vicinity") ,       ("Light squall"),        ("Moderate squall"),        ("Heavy squall"),        "??",                        "??",                           ("Partial squall"),        ("Thunderous squall"),        ("Blowing squall"),        "??",                           ("Drifting squall"),        ("Freezing squall")        },
/* SANDSTORM     */ {("Sandstorm"),             ("Sandstorm in the vicinity") ,    ("Light standstorm"),    ("Moderate sandstorm"),     ("Heavy sandstorm"),     ("Shallow sandstorm"),     "??",                           ("Partual sandstorm"),     ("Thunderous sandstorm"),     ("Blowing sandstorm"),     "??",                           ("Drifting sandstorm"),     ("Freezing sandstorm")     },
/* DUSTSTORM     */ {("Duststorm"),             ("Duststorm in the vicinity") ,    ("Light duststorm"),     ("Moderate duststorm"),     ("Heavy duststorm"),     ("Shallow duststorm"),     "??",                           ("Partial duststorm"),     ("Thunderous duststorm"),     ("Blowing duststorm"),     "??",                           ("Drifting duststorm"),     ("Freezing duststorm")     },
/* FUNNEL_CLOUD  */ {("Funnel cloud"),          ("Funnel cloud in the vicinity") , ("Light funnel cloud"),  ("Moderate funnel cloud"),  ("Thick funnel cloud"),  ("Shallow funnel cloud"),  ("Patches of funnel clouds"), ("Partial funnel clouds"), "??",                           ("Funnel cloud w/ wind"),  "??",                           ("Drifting funnel cloud"),  "??"                         },
/* TORNADO       */ {("Tornado"),               ("Tornado in the vicinity") ,      "??",                      ("Moderate tornado"),       ("Raging tornado"),      "??",                        "??",                           ("Partial tornado"),       ("Thunderous tornado"),       ("Tornado"),               "??",                           ("Drifting tornado"),       ("Freezing tornado")       },
/* DUST_WHIRLS   */ {("Dust whirls"),           ("Dust whirls in the vicinity") ,  ("Light dust whirls"),   ("Moderate dust whirls"),   ("Heavy dust whirls"),   ("Shallow dust whirls"),   ("Patches of dust whirls"),   ("Partial dust whirls"),   "??",                           ("Blowing dust whirls"),   "??",                           ("Drifting dust whirls"),   "??"                         }
};

char *winddir_str[] = {
    ("None"), ("Variable"),
    ("North"), ("North - NorthEast"), ("Northeast"), ("East - NorthEast"),
    ("East"), ("East - Southeast"), ("Southeast"), ("South - Southeast"),
    ("South"), ("South - Southwest"), ("Southwest"), ("West - Southwest"),
    ("West"), ("West - Northwest"), ("Northwest"), ("North - Northwest")
};

char _buf[256];
		
char *WWeather::GetLocStr(void)
{
	return prefs->opLocName;
}
		
char *WWeather::GetTimeStr(void)
{
	if(updating)
		sprintf(_buf, " ");
	else
		sprintf(_buf, time_str);
	return _buf;
}

char *WWeather::GetTempStr(void)
{
	if(updating)
		sprintf(_buf, " ");
	else if(temp == VALUE_NONE)
		sprintf(_buf, "?");
	else
		sprintf(_buf, "%i °%s", temp, prefs->opMetric?"C":"F");
	return _buf;
}

char *WWeather::GetDewStr(void)
{
	if(updating)
		sprintf(_buf, " ");
	else if(dew == VALUE_NONE)
		sprintf(_buf, "?");
	else
		sprintf(_buf, "%i °%s", dew, prefs->opMetric?"C":"F");
	return _buf;
}

char *WWeather::GetSkyStr(void)
{
	if(updating)
		sprintf(_buf, " ");
	else
		sprintf(_buf, sky_str[sky]);
	return _buf;
}

char *WWeather::GetCondStr(void)
{
	if(updating)
		sprintf(_buf, " ");
	else
		sprintf(_buf, cond_str[phem][qual]);
	return _buf;
}

char *WWeather::GetWindStr(void)
{
	if(updating)
		sprintf(_buf, " ");
	else if(winddir == WIND_NONE)
		sprintf(_buf, "Calm");
	else
		sprintf(_buf, "%i %s %s", windspeed, prefs->opMetric?"kph":"knots", winddir_str[winddir]);
		
	return _buf;
}


char *WWeather::GetVisStr(void)
{
	if(updating)
		sprintf(_buf, " ");
	else if(vis == VALUE_NONE)
		sprintf(_buf, "?");
	else
		sprintf(_buf, "%0.1f %s", vis, prefs->opMetric?"kilometers":"miles");
	return _buf;
}

char *WWeather::GetPresStr(void)
{
	if(updating)
		sprintf(_buf, " ");
	else if(pres == VALUE_NONE)
		sprintf(_buf, "?");
	else
		sprintf(_buf, "%0.1f %s", pres, prefs->opMetric?"mmHg":"inHg");
	return _buf;
}

char *WWeather::GetHumStr(void)
{
	if(updating)
		sprintf(_buf, " ");
	else if(hum == VALUE_NONE)
		sprintf(_buf, "?");
	else
		sprintf(_buf, "%i%%", hum);
	return _buf;
}

char *WWeather::GetForeStr(void)
{
	return fore;
}

int32 httpThreadFunc(httpThreadData *data)
{
	HTTP_Io http;
	bool done;
	int cur;
	ssize_t size;
	off_t bufsize;
	char *buffer;
	
	//first we get the metar information
	while(http.SetTo(data->metar, B_READ_ONLY) != B_OK)
		snooze(2000000);
	if(data->weather->prefs->opUseProxy)
		http.UseProxy(data->weather->prefs->opProxyServer, data->weather->prefs->opProxyPort);
	//http.GetSize(&bufsize);
	bufsize = 4096;//for some reason GetSize don't work here
	buffer = new char[bufsize + 1];
	done = false;
	cur = 0;
	while(!done)
	{
		size = http.Read(buffer + cur, bufsize - cur);
		if(size > 0)
		{
			cur += size;
			buffer[cur] = 0;
			if(strstr(buffer, "</html>"))
				done = true;
		}
		else
		{
			buffer[cur] = 0;
			done = true;
		}
	}
	//parse it
	parse_metar(data->weather, data->code, buffer);
	delete[] buffer;
	
	//go ahead and display the metar information; parsing the forecast can take a while
	data->handler->Looper()->PostMessage(WV_HTTP_DONE, data->handler);
	
	//now we get the iwin information
	if(http.SetTo(data->iwin, B_READ_ONLY) != B_OK)
		snooze(2000000);
		
	if(data->weather->prefs->opUseProxy)
		http.UseProxy(data->weather->prefs->opProxyServer, data->weather->prefs->opProxyPort);
	http.GetSize(&bufsize);
	//alert("buffer size: %i", bufsize);
	buffer = new char[bufsize + 1];
	done = false;
	cur = 0;
	while(!done)
	{
		size = http.Read(buffer + cur, bufsize - cur);
		if(size > 0)
		{
			cur += size;
			buffer[cur] = 0;
			if(strstr(buffer, "</html>"))
				done = true;
		}
		else
		{
			buffer[cur] = 0;
			done = true;
		}
	}
	
	delete[] data->weather->fore;
	parse_iwin(data->weather, data->zone, buffer);
	delete[] buffer;
	
	data->handler->Looper()->PostMessage(WV_HTTP_DONE, data->handler);
	
	delete data;
	return 1;
}

void parse_metar(WWeather *weather, char *code, char *string)
{
	char token[32];
	sprintf(token, "\n%s", code);
	string = strstr(string, token);
	if(!string) return;
	string += 6;
	//look for double return which ends the metar section
	char *end = strstr(string, "\n\n");
	if(end) *end = 0;
	
	weather->sem.get();
	
	char *tok = strtok(string, " \n");
	while(tok)
	{
		if(metar_tok_time(weather, tok))
			;
		else if(metar_tok_temp(weather, tok))
			;
		else if(metar_tok_sky(weather, tok))
			;
		else if(metar_tok_cond(weather, tok))
			;
		else if(metar_tok_wind(weather, tok))
			;
		else if(metar_tok_vis(weather, tok))
			;
		else if(metar_tok_pres(weather, tok))
			;
		else
			printf("unknown metar token \"%s\"\n", tok);
		tok = strtok(0, " \n");
	}
	
	//calc hum
    double esat, esurf;
    esat = 6.11 * pow(10.0, (7.5 * weather->temp) / (237.7 + weather->temp));
    esurf = 6.11 * pow(10.0, (7.5 * weather->dew) / (237.7 + weather->dew));
    weather->hum = (int)((esurf/esat) * 100.0);
    
	//translate to english system if god wills it
	if(!weather->prefs->opMetric)
	{
		if(weather->temp != VALUE_NONE) weather->temp = weather->temp*9/5 + 32;
		if(weather->temp != VALUE_NONE) weather->dew = weather->dew*9/5 + 32;
	}
	else
	{
		if(weather->windspeed != VALUE_NONE) weather->windspeed = (int)(weather->windspeed*1.851965);
		if(weather->vis != VALUE_NONE) weather->vis = weather->vis*1.609344;
		if(weather->pres != VALUE_NONE) weather->pres = weather->pres*25.4;
	}
	
	weather->sem.rel();
}

bool metar_tok_time(WWeather *weather, char *tok)
{
	char sday[3], shr[3], smin[3];
	int day, hr, min;
	
	if(regexec(&weather->metar_re[TIME_RE], tok, 0, NULL, 0) == REG_NOMATCH)
		return false;
	
	strncpy(sday, tok, 2);
	sday[2] = 0;
	day = atoi(sday);
	
	strncpy(shr, tok+2, 2);
	shr[2] = 0;
	hr = atoi(shr);
	
	strncpy(smin, tok+4, 2);
	smin[2] = 0;
	min = atoi(smin);
	
	time_t now;
	struct tm *tm;
	
	now = time(NULL);
	//tm = localtime(&now);
	tm = localtime(&now);
	
	/* Estimate timezone */
	/*if(gtm->tm_mday == ltm->tm_mday)
		hour_diff = gtm->tm_hour - ltm->tm_hour;
    else
		if((gtm->tm_mday == ltm->tm_mday + 1) || ((gtm->tm_mday == 1) && (ltm->tm_mday >= 27)))
			hour_diff = gtm->tm_hour + (24 - ltm->tm_hour);
		else
			hour_diff = -((24 - gtm->tm_hour) + ltm->tm_hour);*/
	
	//alert("http: %i\ntime: %i\ngmtoff: %i", hr, tm->tm_hour, tm->tm_gmtoff/3600);
	//alert("%i", timezone);

	tm->tm_mday = day;
	tm->tm_hour = hr + tm->tm_gmtoff/3600;
	if(tm->tm_hour < 0)
		tm->tm_hour += 24;
	tm->tm_min = min;
	
	strftime(weather->time_str, sizeof(weather->time_str), "%l:%M %P"/* %B %e"*/, tm);
	//strftime(weather->time_str, sizeof(weather->time_str), "%l:%M %P %B %e", ltm);
	//sprintf(weather->time_str, "to do");
	if(weather->time_str[0] == ' ')
	{
		for(uint i=0; i<strlen(weather->time_str); i++)
			weather->time_str[i] = weather->time_str[i+1];
	}
	//strcat(weather->time_str, " (GMT)");
	
	return true;
}

bool metar_tok_temp(WWeather *weather, char *tok)
{
	if(regexec(&weather->metar_re[TEMP_RE], tok, 0, NULL, 0) == REG_NOMATCH)
		return false;
	
	char *temp, *dew;
	temp = tok;
	dew = strchr(tok, '/');
	*dew = 0;
	dew++;
	
	if(*temp == 'M')
		weather->temp = -atoi(temp+1);
	else
		weather->temp = atoi(temp);
	
	if(*dew == 'M')
		weather->dew = -atoi(dew+1);
	else
		weather->dew = atoi(dew);
	
	return true;
}

bool metar_tok_sky(WWeather *weather, char *tok)
{
	char stype[4], salt[4];
	int alt = -1;
	
	if(regexec(&weather->metar_re[CLOUD_RE], tok, 0, NULL, 0) == REG_NOMATCH)
		return false;
	
	strncpy(stype, tok, 3);
	stype[3] = 0;
	if(strlen(tok) == 6)
	{
		strncpy(salt, tok+3, 3);
		salt[3] = 0;
		alt = atoi(salt);  /* Altitude - currently unused */
	}
	
	if(!strcmp(stype, "CLR"))
		weather->sky = SKY_CLEAR;
	else if(!strcmp(stype, "BKN"))
		weather->sky = SKY_BROKEN;
	else if(!strcmp(stype, "SCT"))
		weather->sky = SKY_SCATTERED;
	else if(!strcmp(stype, "FEW"))
		weather->sky = SKY_FEW;
	else if(!strcmp(stype, "OVC"))
		weather->sky = SKY_OVERCAST;
	
	return true;
}

bool metar_tok_cond(WWeather *weather, char *tok)
{
	char squal[3], sphen[4];
	char *pphen;
	
	if(regexec(&weather->metar_re[COND_RE], tok, 0, NULL, 0) == REG_NOMATCH)
		return false;
	
	if((strlen(tok) > 3) && ((*tok == '+') || (*tok == '-')))
		++tok;   /* FIX */
	
	if((*tok == '+') || (*tok == '-'))
		pphen = tok + 1;
	else if(strlen(tok) < 4)
		pphen = tok;
	else
		pphen = tok + 2;
	
	memset(squal, 0, sizeof(squal));
	strncpy(squal, tok, pphen - tok);
	squal[pphen - tok] = 0;
	
	memset(sphen, 0, sizeof(sphen));
	strcpy(sphen, pphen);
	
	/* Defaults */
	weather->qual = QUALIFIER_NONE;
	weather->phem = PHENOMENON_NONE;
	
	if(!strcmp(squal, ""))
		weather->qual = QUALIFIER_MODERATE;
	else if(!strcmp(squal, "-"))
		weather->qual = QUALIFIER_LIGHT;
	else if(!strcmp(squal, "+"))
		weather->qual = QUALIFIER_HEAVY;
	else if(!strcmp(squal, "VC"))
		weather->qual = QUALIFIER_VICINITY;
	else if(!strcmp(squal, "MI"))
		weather->qual = QUALIFIER_SHALLOW;
	else if(!strcmp(squal, "BC"))
		weather->qual = QUALIFIER_PATCHES;
	else if(!strcmp(squal, "PR"))
		weather->qual = QUALIFIER_PARTIAL;
	else if(!strcmp(squal, "TS"))
		weather->qual = QUALIFIER_THUNDERSTORM;
	else if(!strcmp(squal, "BL"))
		weather->qual = QUALIFIER_BLOWING;
	else if(!strcmp(squal, "SH"))
		weather->qual = QUALIFIER_SHOWERS;
	else if(!strcmp(squal, "DR"))
		weather->qual = QUALIFIER_DRIFTING;
	else if(!strcmp(squal, "FZ"))
		weather->qual = QUALIFIER_FREEZING;
	
	if(!strcmp(sphen, "DZ"))
		weather->phem = PHENOMENON_DRIZZLE;
	else if(!strcmp(sphen, "RA"))
		weather->phem = PHENOMENON_RAIN;
	else if(!strcmp(sphen, "SN"))
		weather->phem = PHENOMENON_SNOW;
	else if(!strcmp(sphen, "SG"))
		weather->phem = PHENOMENON_SNOW_GRAINS;
	else if(!strcmp(sphen, "IC"))
		weather->phem = PHENOMENON_ICE_CRYSTALS;
	else if(!strcmp(sphen, "PE"))
		weather->phem = PHENOMENON_ICE_PELLETS;
	else if(!strcmp(sphen, "GR"))
		weather->phem = PHENOMENON_HAIL;
	else if(!strcmp(sphen, "GS"))
		weather->phem = PHENOMENON_SMALL_HAIL;
	else if(!strcmp(sphen, "UP"))
		weather->phem = PHENOMENON_UNKNOWN_PRECIPITATION;
	else if(!strcmp(sphen, "BR"))
		weather->phem = PHENOMENON_MIST;
	else if(!strcmp(sphen, "FG"))
		weather->phem = PHENOMENON_FOG;
	else if(!strcmp(sphen, "FU"))
		weather->phem = PHENOMENON_SMOKE;
	else if(!strcmp(sphen, "VA"))
		weather->phem = PHENOMENON_VOLCANIC_ASH;
	else if(!strcmp(sphen, "SA"))
		weather->phem = PHENOMENON_SAND;
	else if(!strcmp(sphen, "HZ"))
		weather->phem = PHENOMENON_HAZE;
	else if(!strcmp(sphen, "PY"))
		weather->phem = PHENOMENON_SPRAY;
	else if(!strcmp(sphen, "DU"))
		weather->phem = PHENOMENON_DUST;
	else if(!strcmp(sphen, "SQ"))
		weather->phem = PHENOMENON_SQUALL;
	else if(!strcmp(sphen, "SS"))
		weather->phem = PHENOMENON_SANDSTORM;
	else if(!strcmp(sphen, "DS"))
		weather->phem = PHENOMENON_DUSTSTORM;
	else if(!strcmp(sphen, "PO"))
		weather->phem = PHENOMENON_DUST_WHIRLS;
	else if(!strcmp(sphen, "+FC"))
		weather->phem = PHENOMENON_TORNADO;
	else if(!strcmp(sphen, "FC"))
		weather->phem = PHENOMENON_FUNNEL_CLOUD;
	
	return true;
}

bool metar_tok_wind(WWeather *weather, char *tok)
{
	char sdir[4], sspd[4], sgust[4];
	int dir, spd, gust = -1;
	char *gustp;
	
	if(regexec(&weather->metar_re[WIND_RE], tok, 0, NULL, 0) == REG_NOMATCH)
		return false;
	
	strncpy(sdir, tok, 3);
	sdir[3] = 0;
	dir = (!strcmp(sdir, "VRB")) ? -1 : atoi(sdir);
	
	memset(sspd, 0, sizeof(sspd));
	strncpy(sspd, tok+3, strspn(tok+3, CONST_DIGITS));
	spd = atoi(sspd);
	
	gustp = strchr(tok, 'G');
	if(gustp)
	{
		memset(sgust, 0, sizeof(sgust));
		strncpy(sgust, gustp+1, strspn(gustp+1, CONST_DIGITS));
		gust = atoi(sgust);
	}
	
	if((349 <= dir) && (dir <= 11))
		weather->winddir = WIND_N;
	else if((12 <= dir) && (dir <= 33))
		weather->winddir = WIND_NNE;
	else if((34 <= dir) && (dir <= 56))
		weather->winddir = WIND_NE;
	else if((57 <= dir) && (dir <= 78))
		weather->winddir = WIND_ENE;
	else if((79 <= dir) && (dir <= 101))
		weather->winddir = WIND_E;
	else if((102 <= dir) && (dir <= 123))
		weather->winddir = WIND_ESE;
	else if((124 <= dir) && (dir <= 146))
		weather->winddir = WIND_SE;
	else if((147 <= dir) && (dir <= 168))
		weather->winddir = WIND_SSE;
	else if((169 <= dir) && (dir <= 191))
		weather->winddir = WIND_S;
	else if((192 <= dir) && (dir <= 213))
		weather->winddir = WIND_SSW;
	else if((214 <= dir) && (dir <= 236))
		weather->winddir = WIND_SW;
	else if((247 <= dir) && (dir <= 258))
		weather->winddir = WIND_WSW;
	else if((259 <= dir) && (dir <= 281))
		weather->winddir = WIND_W;
	else if((282 <= dir) && (dir <= 303))
		weather->winddir = WIND_WNW;
	else if((304 <= dir) && (dir <= 326))
		weather->winddir = WIND_NW;
	else if((327 <= dir) && (dir <= 348))
		weather->winddir = WIND_NNW;
	
	weather->windspeed = spd;
	
	return true;
}

bool metar_tok_vis(WWeather *weather, char *tok)
{
	char *pfrac, *pend;
	char sval[4];
	int val;
	
	if(regexec(&weather->metar_re[VIS_RE], tok, 0, NULL, 0) == REG_NOMATCH)
		return false;
	
	pfrac = strchr(tok, '/');
	pend = strstr(tok, "SM");
	memset(sval, 0, sizeof(sval));
	
	if(pfrac)
	{
		strncpy(sval, pfrac + 1, pend - pfrac - 1);
		val = atoi(sval);
		weather->vis = (*tok == 'M') ? 0.001 : (1.0 / val);
	}
	else
	{
		strncpy(sval, tok, pend - tok);
		val = atoi(sval);
		weather->vis = val;
	}
	
	return true;
}

bool metar_tok_pres(WWeather *weather, char *tok)
{
	if(regexec(&weather->metar_re[PRES_RE], tok, 0, NULL, 0) == REG_NOMATCH)
		return false;
	
	if(*tok == 'A')
	{
		char sintg[3], sfract[3];
		int intg, fract;
		
		strncpy(sintg, tok+1, 2);
		sintg[2] = 0;
		intg = atoi(sintg);
		
		strncpy(sfract, tok+3, 2);
		sfract[2] = 0;
		fract = atoi(sfract);
		
		weather->pres = intg + (fract/100.0);
	}
	else
	{  /* *tokp == 'Q' */
		char spres[5];
		int pres;
		
		strncpy(spres, tok+1, 4);
		spres[4] = 0;
		pres = atoi(spres);
		
		weather->pres = pres*0.02963742;
	}
	
	return true;
}

void parse_iwin(WWeather *weather, char *zone, char *string)
{
	char *p, *rangep = NULL;
	regmatch_t match[1];
	int ret;
	
	p = string;
	while((ret = regexec(&weather->iwin_re, p, 1, match, 0)) != REG_NOMATCH)
	{
		rangep = p + match[0].rm_so;
		
		p = strchr(rangep, '\n');
		if(iwin_range_match(rangep, zone))
			break;
	}
	
	if(ret != REG_NOMATCH)
	{
		char *end = strstr(p, "\n</PRE>");
		if((regexec(&weather->iwin_re, p, 1, match, 0) != REG_NOMATCH) && (end - p > match[0].rm_so))
			end = p + match[0].rm_so - 1;
		*end = 0;
		
		weather->fore = new char[strlen(rangep) + 1];
		strcpy(weather->fore, rangep);
	}
	else
	{
		//char *error = "Forecast not available";
		char *error = "Forecast not available in this area.\n\nForecasts are only available in some parts of the united states.";
		weather->fore = new char[strlen(error) + 1];
		strcpy(weather->fore, error);
	}
	
	//from here on out it's all cosmetic to make it not look like human defication

	//end it proporly
	char *end = strstr(weather->fore, "\r\n\r\n$$");
	if(end)
		*end = 0;
	end = strstr(weather->fore, "\r\n\r\n=\r\n$$");
	if(end)
		*end = 0;

//translate removes from and adds to from weather->fore
#define TRANSLATE(from, to) \
{\
	char *bad = strstr(weather->fore, from);\
	while(bad)\
	{\
		strncpy(bad, to, strlen(to));\
		bad += strlen(to);\
		for(uint i=0; i<strlen(bad) + strlen(to) - strlen(from) + 1; i++)\
			bad[i] = bad[i + strlen(from) - strlen(to)];\
		bad = strstr(bad, from);\
	}\
}
	TRANSLATE("\r\n\r\n", "\n\n");
	TRANSLATE("\r\n", " ");
	TRANSLATE("\n...", "\n");
	TRANSLATE("\n.", "\n");
	TRANSLATE("...", ": ");
	TRANSLATE(" .", "\n");
	
	TRANSLATE(" -", "-");
	TRANSLATE("- ", "-");
	TRANSLATE("  ", " ");
#undef TRANSLATE
	
	//there are these anoying tables of temperatures that some times get reported
	//they are ugly, and I don't want to have to deal with them, so we strip them out.
	//they start with < and last an entire line. find and kill that line.
	end = strchr(weather->fore, '<');
	if(end)
	{
		char *start = end;
		end = strchr(start, '\n');
		int skip = end - start + 1;
		for(int i=0; i<int(strlen(start)) - skip + 1; i++)
			start[i] = start[i + skip];
	}
	
	//make it lowercase in preperation for upper-casing certain things
	for(uint i=0; i<strlen(weather->fore); i++)
		weather->fore[i] = tolower(weather->fore[i]);

//translate now makes the character AFTER start uppercase
#define TRANSLATE(start) \
{\
	char *bad = strstr(weather->fore, start);\
	while(bad)\
	{\
		*(bad + strlen(start)) = toupper(*(bad + strlen(start)));\
		bad += strlen(start);\
		bad = strstr(bad, start);\
	}\
}
	weather->fore[0] = toupper(weather->fore[0]);
		//first char should be caped too
	TRANSLATE("\n");
	TRANSLATE("-");
	TRANSLATE(": ");
	TRANSLATE(". ");
#undef TRANSLATE
}

bool iwin_range_match(char *range, char *zone)
{
	char *zonep;
	char *endp;
	char zone_state[4], zone_num_str[4];
	int zone_num;
	
	endp = range + strcspn(range, " \t\r\n") - 1;
	if (strspn(endp - 6, CONST_DIGITS) == 6)
	{
		--endp;
		while (*endp != '-')
			--endp;
	}
	
	strncpy(zone_state, zone, 3);
	zone_state[3] = 0;
	strncpy(zone_num_str, zone + 3, 3);
	zone_num_str[3] = 0;
	zone_num = atoi(zone_num_str);
	
	zonep = range;
	while (zonep < endp)
	{
		char from_str[4], to_str[4];
		int from, to;
		
		if (strncmp(zonep, zone_state, 3) != 0)
		{
			zonep += 3;
			zonep += strcspn(zonep, CONST_ALPHABET "\n");
			continue;
		}
		
		zonep += 3;
		
		do
		{
			strncpy(from_str, zonep, 3);
			from_str[3] = 0;
			from = atoi(from_str);
			
			zonep += 3;
			
			if(*zonep == '-')
			{
				++zonep;
				to = from;
			}
			else if(*zonep == '>')
			{
				++zonep;
				strncpy(to_str, zonep, 3);
				to_str[3] = 0;
				to = atoi(to_str);
				zonep += 4;
			}
			else
			{
				to = 0;
			}
		
		if((from <= zone_num) && (zone_num <= to))
			return true;
		
		}while(!isupper(*zonep) && (zonep < endp));
	}
	
	return false;
}