#ifndef _weather_h_
#define _weather_h_

#include "regex.h"
#include "Sem.h"
#include <OS.h>

class WPrefs;
class BHandler;
class BBitmap;

enum
{
	TIME_RE = 0,
	WIND_RE,
	VIS_RE,
	CLOUD_RE,
	TEMP_RE,
	PRES_RE,
	COND_RE,
	RE_NUM
};

#define TIME_RE_STR  "^([0-9]{6})Z$"
#define WIND_RE_STR  "^(([0-9]{3})|VRB)([0-9]?[0-9]{2})(G[0-9]?[0-9]{2})?KT$"
#define VIS_RE_STR   "^(([0-9]?[0-9])|(M?1/[0-9]?[0-9]))SM$"
#define CLOUD_RE_STR "^(CLR|BKN|SCT|FEW|OVC)([0-9]{3})?$"
#define TEMP_RE_STR  "^(M?[0-9][0-9])/(M?[0-9][0-9])$"
#define PRES_RE_STR  "^(A|Q)([0-9]{4})$"
#define COND_RE_STR  "^(-|\\+)?(VC|MI|BC|PR|TS|BL|SH|DR|FZ)?(DZ|RA|SN|SG|IC|PE|GR|GS|UP|BR|FG|FU|VA|SA|HZ|PY|DU|SQ|SS|DS|PO|\\+?FC)$"
#define IWIN_RE_STR "([A-Z][A-Z]Z(([0-9]{3}>[0-9]{3}-)|([0-9]{3}-))+)+([0-9]{6}-)?"

#define CONST_ALPHABET "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
#define CONST_DIGITS "0123456789"

#define VALUE_NONE 0xfdfd

enum SKY
{
	SKY_NONE,
	SKY_CLEAR,
	SKY_BROKEN,
	SKY_SCATTERED,
	SKY_FEW,
	SKY_OVERCAST
};

enum PHENOMENON
{
	PHENOMENON_NONE,
	
	PHENOMENON_DRIZZLE,
	PHENOMENON_RAIN,
	PHENOMENON_SNOW,
	PHENOMENON_SNOW_GRAINS,
	PHENOMENON_ICE_CRYSTALS,
	PHENOMENON_ICE_PELLETS,
	PHENOMENON_HAIL,
	PHENOMENON_SMALL_HAIL,
	PHENOMENON_UNKNOWN_PRECIPITATION,
	
	PHENOMENON_MIST,
	PHENOMENON_FOG,
	PHENOMENON_SMOKE,
	PHENOMENON_VOLCANIC_ASH,
	PHENOMENON_SAND,
	PHENOMENON_HAZE,
	PHENOMENON_SPRAY,
	PHENOMENON_DUST,
	
	PHENOMENON_SQUALL,
	PHENOMENON_SANDSTORM,
	PHENOMENON_DUSTSTORM,
	PHENOMENON_FUNNEL_CLOUD,
	PHENOMENON_TORNADO,
	PHENOMENON_DUST_WHIRLS
};

enum QUALIFIER
{
	QUALIFIER_NONE,
	
	QUALIFIER_VICINITY,
	
	QUALIFIER_LIGHT,
	QUALIFIER_MODERATE,
	QUALIFIER_HEAVY,
	QUALIFIER_SHALLOW,
	QUALIFIER_PATCHES,
	QUALIFIER_PARTIAL,
	QUALIFIER_THUNDERSTORM,
	QUALIFIER_BLOWING,
	QUALIFIER_SHOWERS,
	QUALIFIER_DRIFTING,
	QUALIFIER_FREEZING
};

enum WINDDIR
{
    WIND_NONE, WIND_VARIABLE,
    WIND_N, WIND_NNE, WIND_NE, WIND_ENE,
    WIND_E, WIND_ESE, WIND_SE, WIND_SSE,
    WIND_S, WIND_SSW, WIND_SW, WIND_WSW,
    WIND_W, WIND_WNW, WIND_NW, WIND_NNW,
};

enum
{
	RES_BLANK = 0,
	RES_UPDATE = 2,
	RES_SUN = 4,
	RES_PART = 6,
	RES_CLOUD = 8,
	RES_RAIN = 10,
	RES_SNOW = 12,
	RES_LIGHT = 14
};

class WWeather
{
	public:
		WWeather(WPrefs *prefs, BHandler *ihandler);
		~WWeather(void);
		void BlankInfo(void);
		BBitmap *ChooseBitmap(void);
		void Update(void);
		
		char *GetLocStr(void);
		char *GetTimeStr(void);
		char *GetTempStr(void);
		char *GetDewStr(void);
		char *GetSkyStr(void);
		char *GetCondStr(void);
		char *GetWindStr(void);
		char *GetVisStr(void);
		char *GetPresStr(void);
		char *GetHumStr(void);
		char *GetForeStr(void);
	
	public:
		bool updating;//this is only user initiated updating, not pulses
		WSem sem;
		regex_t metar_re[RE_NUM], iwin_re;
		WPrefs *prefs;
		BHandler *handler;
		
		char time_str[256];
		int temp, dew;
		int sky;
		int phem;
		int qual;
		int winddir;
		int windspeed;
		double vis;
		double pres;
		int hum;
		char *fore;
};

#endif